==> drive-0.0.1-2.0.pre.tar.gz: lib/drive_driver.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math' as math;

import 'package:stack_trace/stack_trace.dart';
import 'package:flutter_driver/flutter_driver.dart';
import 'common.dart';

bool _isWeb = false;
bool _running = true;

Set<String> _groupVisited = <String>{};
Map<String, Map<String, dynamic>> _tests = {};
Map<String, Map<String, dynamic>> _failedTestEvents = {};

String _boldText(String text) {
  return '\u001b[1m$text\u001b[0m';
}

String _grayText(String text) {
  return '\u001b[1;30m$text\u001b[0m';
}

String _greenText(String text) {
  return '\u001b[32m$text\u001b[0m';
}

String _redText(String text) {
  return '\x1b[1;31m$text\u001b[0m';
}

void _handleSuiteStartEvent(Map<String, dynamic> eventData) {
  List<dynamic> tests = eventData[kTests];

  tests.forEach((test) {
    Map<String, dynamic> testMap = {};
    testMap.addAll(test as Map);
    _tests[testMap[kId]] = testMap;
  });

  _isWeb = eventData[kWeb] as bool;

  print('');
  print('');
  print('------------------------------');
  print('|         Drive Tests        |');
  print('------------------------------');
  print('');
  print('Running suite with ${tests.length} tests...');
  print('');
}

void _handleTestStartEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kId];
  Map<String, dynamic> test = _tests[testId];

  String groupId = test[kGroupId];
  String groupName = test[kGroupName];

  if (groupId != null && !_groupVisited.contains(groupId)) {
    print('\n  ${_boldText(groupName)}');
    _groupVisited.add(groupId);
  }
}

void _handleTestCompleteEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kId];
  bool passed = eventData[kPassed];
  Map<String, dynamic> test = _tests[testId];
  String testName = test[kName];

  if (passed) {
    stdout.writeln(_greenText('     ✔ ') + testName);
  } else {
    String error = eventData[kError];
    stdout.writeln(_redText('     ✘ $testName'));
    Iterable<String> errorLines = error.split('\n').take(2);
    bool firstLine = false;
    errorLines.forEach((line) {
      if (!firstLine) {
        stdout.writeln(_redText('       [E]: ${line}'));
        firstLine = true;
      } else {
        stdout.writeln(_redText('            ${line}'));
      }
    });
    _failedTestEvents[testId] = eventData;
  }
  stdout.writeln('');
}

void _handlePrintEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kTestId];
  String message = eventData[kMessage];
  if (testId != "0") {
    // for a specific test so indent to match test indentation
    print('       ${_boldText('[LOG]')}: ${_grayText(message)}');
  } else {
    print('  ${_boldText('[LOG]')}: ${_grayText(message)}');
  }
}

Future<void> main() async {
  final FlutterDriver driver = await FlutterDriver.connect();
  await Future.delayed(Duration(seconds: 1));

  while (_running) {
    String eventString =
        await driver.requestData(null, timeout: const Duration(minutes: 1));
    if (eventString == null || eventString.isEmpty) {
      await Future.delayed(Duration(milliseconds: 50));
      continue;
    }

    Map<String, dynamic> event = json.decode(eventString);
    String eventAction = event[kAction];
    Map<String, dynamic> eventData = event[kData];

    switch (eventAction) {
      case EventActionError:
        {
          _running = false;
          // TODO better log output
          print('An error occurred outside your tests...');
          print(eventData[kError]);
          print(eventData[kStacktrace]);
          await driver.close();
          exit(1);
        }
        break;
      case EventActionSuiteStart:
        _handleSuiteStartEvent(eventData);
        break;
      case EventActionSuiteComplete:
        _running = false;
        break;
      case EventActionTestStart:
        _handleTestStartEvent(eventData);
        break;
      case EventActionTestComplete:
        _handleTestCompleteEvent(eventData);
        break;
      case EventActionPrint:
        _handlePrintEvent(eventData);
        break;
    }
  }

  await driver.close();

  print('');
  if (_failedTestEvents.isEmpty) {
    print('------------------------------');
    print('');
    stdout.writeln(_greenText(' ✔ ') + 'all tests are passing.');
    print('');
    exit(0);
  } else {
    print('');
    print('------------------------------');
    print('|        Test Failures       |');
    print('------------------------------');
    print('');
    int failureCount = 1;
    _failedTestEvents.values.forEach((testFailureEvent) {
      String groupName = testFailureEvent[kGroupName];
      String testName = testFailureEvent[kName];

      if (groupName != null) {
        stdout.writeln('$failureCount) $groupName > $testName');
      } else {
        stdout.writeln('$failureCount) $testName');
      }

      String error = testFailureEvent[kError];
      StackTrace stacktrace = StackTrace.fromString(
          testFailureEvent[kStacktrace]
              .replaceAll(Directory.current.path.replaceFirst('/', ''), '.'));
      List<Frame> stackFrames = Trace.from(stacktrace).terse.frames;

      bool firstLine = false;
      Iterable<String> errorLines = error.split('\n').take(2);
      errorLines.forEach((line) {
        if (!firstLine) {
          stdout.writeln(_redText('   [E]: $line'));
          firstLine = true;
        } else {
          stdout.writeln(_redText('        $line'));
        }
      });

      var longest =
          stackFrames.map((frame) => frame.location.length).fold(0, math.max);
      stackFrames.take(10).forEach((frame) {
        String location = frame.location.replaceAll(' ', ':');
        location = location.padRight(longest);
        String member = frame.member;

        if (frame.isCore ||
            location.contains('package:test_api') ||
            location.contains('package:flutter_test') ||
            location.contains('package:stack_trace')) {
          stdout.writeln(_grayText('          ├─  ${location} $member'));
        } else {
          stdout.writeln(_boldText('          ├─  ${location} $member'));
        }
      });
      print('');
      print('');
      failureCount++;
    });
    exit(_failedTestEvents.length);
  }
}

==> drive-0.0.1-3.0.pre.tar.gz: lib/drive_driver.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math' as math;

import 'package:stack_trace/stack_trace.dart';
import 'package:flutter_driver/flutter_driver.dart';
import 'common.dart';

bool _isWeb = false;
bool _running = true;

Set<String> _groupVisited = <String>{};
Map<String, Map<String, dynamic>> _tests = {};
Map<String, Map<String, dynamic>> _failedTestEvents = {};

String _boldText(String text) {
  return '\u001b[1m$text\u001b[0m';
}

String _grayText(String text) {
  return '\u001b[1;30m$text\u001b[0m';
}

String _greenText(String text) {
  return '\u001b[32m$text\u001b[0m';
}

String _redText(String text) {
  return '\x1b[1;31m$text\u001b[0m';
}

void _handleSuiteStartEvent(Map<String, dynamic> eventData) {
  List<dynamic> tests = eventData[kTests];

  tests.forEach((test) {
    Map<String, dynamic> testMap = {};
    testMap.addAll(test as Map);
    _tests[testMap[kId]] = testMap;
  });

  _isWeb = eventData[kWeb] as bool;

  print('');
  print('');
  print('------------------------------');
  print('|         Drive Tests        |');
  print('------------------------------');
  print('');
  print('Running suite with ${tests.length} tests...');
  print('');
}

void _handleTestStartEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kId];
  Map<String, dynamic> test = _tests[testId];

  String groupId = test[kGroupId];
  String groupName = test[kGroupName];

  if (groupId != null && !_groupVisited.contains(groupId)) {
    print('\n\n  ${_boldText(groupName)}');
    _groupVisited.add(groupId);
  }
}

void _handleTestCompleteEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kId];
  bool passed = eventData[kPassed];
  Map<String, dynamic> test = _tests[testId];
  String testName = test[kName];

  if (passed) {
    stdout.writeln(_greenText('     ✔ ') + testName);
  } else {
    String error = eventData[kError];
    stdout.writeln(_redText('     ✘ $testName'));
    Iterable<String> errorLines = error.split('\n').take(2);
    bool firstLine = false;
    errorLines.forEach((line) {
      if (!firstLine) {
        stdout.writeln(_redText('       [E]: ${line}'));
        firstLine = true;
      } else {
        stdout.writeln(_redText('            ${line}'));
      }
    });
    _failedTestEvents[testId] = eventData;
  }
}

void _handlePrintEvent(Map<String, dynamic> eventData) {
  String testId = eventData[kTestId];
  String message = eventData[kMessage];
  if (testId != "0") {
    // for a specific test so indent to match test indentation
    print('       ${_boldText('[LOG]')}: ${_grayText(message)}');
  } else {
    print('  ${_boldText('[LOG]')}: ${_grayText(message)}');
  }
}

Future<void> main() async {
  final FlutterDriver driver = await FlutterDriver.connect();
  await Future.delayed(Duration(seconds: 1));

  while (_running) {
    String eventString =
        await driver.requestData(null, timeout: const Duration(minutes: 1));
    if (eventString == null || eventString.isEmpty) {
      await Future.delayed(Duration(milliseconds: 50));
      continue;
    }

    Map<String, dynamic> event = json.decode(eventString);
    String eventAction = event[kAction];
    Map<String, dynamic> eventData = event[kData];

    switch (eventAction) {
      case EventActionError:
        {
          _running = false;
          // TODO better log output
          print('An error occurred outside your tests...');
          print(eventData[kError]);
          print(eventData[kStacktrace]);
          await driver.close();
          exit(1);
        }
        break;
      case EventActionSuiteStart:
        _handleSuiteStartEvent(eventData);
        break;
      case EventActionSuiteComplete:
        _running = false;
        break;
      case EventActionTestStart:
        _handleTestStartEvent(eventData);
        break;
      case EventActionTestComplete:
        _handleTestCompleteEvent(eventData);
        break;
      case EventActionPrint:
        _handlePrintEvent(eventData);
        break;
    }
  }

  await driver.close();

  print('');
  if (_failedTestEvents.isEmpty) {
    print('------------------------------');
    print('');
    stdout.writeln(_greenText(' ✔ ') + 'all tests are passing.');
    print('');
    exit(0);
  } else {
    print('');
    print('------------------------------');
    print('|        Test Failures       |');
    print('------------------------------');
    print('');
    int failureCount = 1;
    _failedTestEvents.values.forEach((testFailureEvent) {
      String groupName = testFailureEvent[kGroupName];
      String testName = testFailureEvent[kName];

      if (groupName != null) {
        stdout.writeln('$failureCount) $groupName > $testName');
      } else {
        stdout.writeln('$failureCount) $testName');
      }

      String error = testFailureEvent[kError];
      StackTrace stacktrace = StackTrace.fromString(
          testFailureEvent[kStacktrace]
              .replaceAll(Directory.current.path.replaceFirst('/', ''), '.'));
      List<Frame> stackFrames = Trace.from(stacktrace).terse.frames;

      bool firstLine = false;
      Iterable<String> errorLines = error.split('\n').take(2);
      errorLines.forEach((line) {
        if (!firstLine) {
          stdout.writeln(_redText('   [E]: $line'));
          firstLine = true;
        } else {
          stdout.writeln(_redText('        $line'));
        }
      });

      var longest =
          stackFrames.map((frame) => frame.location.length).fold(0, math.max);
      stackFrames.take(10).forEach((frame) {
        String location = frame.location.replaceAll(' ', ':');
        location = location.padRight(longest);
        String member = frame.member;

        if (frame.isCore ||
            location.contains('package:test_api') ||
            location.contains('package:flutter_test') ||
            location.contains('package:stack_trace')) {
          stdout.writeln(_grayText('          ├─  ${location} $member'));
        } else {
          stdout.writeln(_boldText('          ├─  ${location} $member'));
        }
      });
      print('');
      print('');
      failureCount++;
    });
    exit(_failedTestEvents.length);
  }
}
